@@ -55,12 +55,26 @@
     prolog:deprecated//1,
     prolog:message_location//1,
     prolog:message_line_element/2)).
+% Lang, Term versions
+:- multifile
+    prolog:message//2,              % entire message
+    prolog:error_message//2,        % 1-st argument of error term
+    prolog:message_context//2,      % Context of error messages
+    prolog:message_location//2,	    % (File) location of error messages
+    prolog:deprecated//2.	    % Deprecated features
+:- '$hide'((
+    prolog:message//2,
+    prolog:error_message//2,
+    prolog:message_context//2,
+    prolog:deprecated//2,
+    prolog:message_location//2)).
 
 :- discontiguous
     prolog_message/3.
 
 :- public
-    translate_message//1.
+    translate_message//1,           % +Message (deprecated)
+    prolog:translate_message//1.    % +Message
 
 :- create_prolog_flag(message_context, [thread], []).
 
@@ -69,33 +83,50 @@
 %   Translate a message Term into message lines. The produced lines
 %   is a list of
 %
-%       * nl
-%       Emit a newline
-%       * Fmt-Args
-%       Emit the result of format(Fmt, Args)
-%       * Fmt
-%       Emit the result of format(Fmt)
-%       * flush
-%       Used only as last element of the list.   Simply flush the
-%       output instead of producing a final newline.
-%       * at_same_line
-%       Start the messages at the same line (instead of using ~N)
+%       - nl
+%         Emit a newline
+%       - Fmt-Args
+%         Emit the result of format(Fmt, Args)
+%       - Fmt
+%         Emit the result of format(Fmt)
+%       - ansi(Code, Fmt, Args)
+%         Use ansi_format/3 for color output.
+%       - flush
+%         Used only as last element of the list.   Simply flush the
+%         output instead of producing a final newline.
+%       - at_same_line
+%         Start the messages at the same line (instead of using ~N)
+%
+%   @deprecated  Use  code  for   message    translation   should   call
+%   prolog:translate_message//1.
+
+prolog:translate_message(Term) -->
+    translate_message(Term).
+
+%!  translate_message(+Term)// is det.
+%
+%   Translate a message term into  message   lines.  This version may be
+%   called from user and library definitions for message translation.
 
 translate_message(Term) -->
+    { nonvar(Term) },
+    (   { message_lang(Lang) },
+        prolog:message(Lang, Term)
+    ;   prolog:message(Term)
+    ),
+    !.
+translate_message(Term) -->
+    { nonvar(Term) },
     translate_message2(Term),
     !.
 translate_message(Term) -->
-    { Term = error(_, _) },
+    { nonvar(Term),
+      Term = error(_, _)
+    },
     [ 'Unknown exception: ~p'-[Term] ].
 translate_message(Term) -->
     [ 'Unknown message: ~p'-[Term] ].
 
-translate_message2(Term) -->
-    {var(Term)},
-    !,
-    [ 'Unknown message: ~p'-[Term] ].
-translate_message2(Term) -->
-    prolog:message(Term).
 translate_message2(Term) -->
     prolog_message(Term).
 translate_message2(error(resource_error(stack), Context)) -->
@@ -123,12 +154,24 @@ make_message_lines([Last],  ['~w'-[Last]|T], T) :- !.
 make_message_lines([L0|LT], ['~w'-[L0],nl|T0], T) :-
     make_message_lines(LT, T0, T).
 
+%!  term_message(+Term)//
+%
+%   Deal  with  the  formal  argument    of  error(Format,  ImplDefined)
+%   exception  terms.  The  `ImplDefined`   argument    is   handled  by
+%   swi_location//2.
+
+:- public term_message//1.
 term_message(Term) -->
     {var(Term)},
     !,
     [ 'Unknown error term: ~p'-[Term] ].
 term_message(Term) -->
-    prolog:error_message(Term).
+    { message_lang(Lang) },
+    prolog:error_message(Lang, Term),
+    !.
+term_message(Term) -->
+    prolog:error_message(Term),
+    !.
 term_message(Term) -->
     iso_message(Term).
 term_message(Term) -->
@@ -484,17 +527,24 @@ cond_location(File:Line) -->
     { file_base_name(File, Base) },
     [ '~w:~d'-[Base, Line] ].
 
+%!  swi_location(+Term)// is det.
+%
+%   Print location information for error(Formal,   ImplDefined) from the
+%   ImplDefined term.
+
+:- public swi_location//1.
 swi_location(X) -->
-    { var(X)
-    },
-    !,
-    [].
+    { var(X) },
+    !.
+swi_location(Context) -->
+    { message_lang(Lang) },
+    prolog:message_location(Lang, Context),
+    !.
 swi_location(Context) -->
     prolog:message_location(Context),
     !.
 swi_location(context(Caller, _Msg)) -->
-    { ground(Caller)
-    },
+    { ground(Caller) },
     !,
     caller(Caller).
 swi_location(file(Path, Line, -1, _CharNo)) -->
@@ -529,11 +579,22 @@ caller(Caller) -->
     [ '~p: '-[Caller] ].
 
 
+%!  swi_extra(+Term)// is det.
+%
+%   Extract information from the  second   argument  of an error(Formal,
+%   ImplDefined) that is printed _after_ the core of the message.
+%
+%   @see swi_location//1 uses the same term   to insert context _before_
+%   the core of the message.
+
 swi_extra(X) -->
-    { var(X)
-    },
+    { var(X) },
     !,
     [].
+swi_extra(Context) -->
+    { message_lang(Lang) },
+    prolog:message_context(Lang, Context),
+    !.
 swi_extra(Context) -->
     prolog:message_context(Context).
 swi_extra(context(_, Msg)) -->
@@ -574,6 +635,46 @@ thread_context -->
                  *        NORMAL MESSAGES       *
                  *******************************/
 
+prolog_message(welcome) -->
+    [ 'Welcome to SWI-Prolog (' ],
+    prolog_message(threads),
+    prolog_message(address_bits),
+    ['version ' ],
+    prolog_message(version),
+    [ ')', nl ],
+    prolog_message(copyright),
+    [ nl ],
+    translate_message(user_versions),
+    [ nl ],
+    prolog_message(documentaton),
+    [ nl, nl ].
+prolog_message(user_versions) -->
+    (   { findall(Msg, prolog:version_msg(Msg), Msgs),
+          Msgs \== []
+        }
+    ->  [nl],
+        user_version_messages(Msgs)
+    ;   []
+    ).
+prolog_message(deprecated(Term)) -->
+    { nonvar(Term) },
+    (   { message_lang(Lang) },
+        prolog:deprecated(Lang, Term)
+    ->  []
+    ;   prolog:deprecated(Term)
+    ->  []
+    ;   deprecated(Term)
+    ).
+prolog_message(unhandled_exception(E)) -->
+    { nonvar(E) },
+    [ 'Unhandled exception: ' ],
+    (   translate_message(E)
+    ->  []
+    ;   [ '~p'-[E] ]
+    ).
+
+%!  prolog_message(+Term)//
+
 prolog_message(initialization_error(_, E, File:Line)) -->
     !,
     [ '~w:~d: '-[File, Line],
@@ -612,12 +713,6 @@ prolog_message(init_goal_failed(Error, Text)) -->
     !,
     [ '-g ~w: '-[Text] ],
     translate_message(Error).
-prolog_message(unhandled_exception(E)) -->
-    [ 'Unhandled exception: ' ],
-    (   translate_message2(E)
-    ->  []
-    ;   [ '~p'-[E] ]
-    ).
 prolog_message(goal_failed(Context, Goal)) -->
     [ 'Goal (~w) failed: ~p'-[Context, Goal] ].
 prolog_message(no_current_module(Module)) -->
@@ -661,8 +756,6 @@ prolog_message(unknown_in_module_user) -->
       'Please use :- dynamic or limit usage of unknown to a module.', nl,
       'See https://www.swi-prolog.org/howto/database.html'
     ].
-prolog_message(deprecated(What)) -->
-    deprecated(What).
 prolog_message(untable(PI)) -->
     [ 'Reconsult: removed tabling for ~p'-[PI] ].
 
@@ -1140,31 +1233,10 @@ prolog_message(copyright) -->
     [ 'SWI-Prolog comes with ABSOLUTELY NO WARRANTY. This is free software.', nl,
       'Please run ?- license. for legal details.'
     ].
-prolog_message(user_versions) -->
-    (   { findall(Msg, prolog:version_msg(Msg), Msgs),
-          Msgs \== []
-        }
-    ->  [nl],
-        user_version_messages(Msgs)
-    ;   []
-    ).
 prolog_message(documentaton) -->
     [ 'For online help and background, visit https://www.swi-prolog.org', nl,
       'For built-in help, use ?- help(Topic). or ?- apropos(Word).'
     ].
-prolog_message(welcome) -->
-    [ 'Welcome to SWI-Prolog (' ],
-    prolog_message(threads),
-    prolog_message(address_bits),
-    ['version ' ],
-    prolog_message(version),
-    [ ')', nl ],
-    prolog_message(copyright),
-    [ nl ],
-    prolog_message(user_versions),
-    [ nl ],
-    prolog_message(documentaton),
-    [ nl, nl ].
 prolog_message(about) -->
     [ 'SWI-Prolog version (' ],
     prolog_message(threads),
@@ -1432,6 +1504,11 @@ history_events([Nr/Event|T]) -->
     history_events(T).
 
 
+%!  user_version_messages(+Terms)//
+%
+%   Helper for the `welcome`  message   to  print information registered
+%   using version/1.
+
 user_version_messages([]) --> [].
 user_version_messages([H|T]) -->
     user_version_message(H),
@@ -1440,7 +1517,7 @@ user_version_messages([H|T]) -->
 %!  user_version_message(+Term)
 
 user_version_message(Term) -->
-    translate_message2(Term), !, [nl].
+    translate_message(Term), !, [nl].
 user_version_message(Atom) -->
     [ '~w'-[Atom], nl ].
 
@@ -1635,7 +1712,7 @@ prolog_message(pack(attached(Pack, BaseDir))) -->
     [ 'Attached package ~w at ~q'-[Pack, BaseDir] ].
 prolog_message(pack(duplicate(Entry, OldDir, Dir))) -->
     [ 'Package ~w already attached at ~q.'-[Entry,OldDir], nl,
-      '\tIgnoring version from ~q'- [Entry, OldDir, Dir]
+      '\tIgnoring version from ~q'- [Dir]
     ].
 prolog_message(pack(no_arch(Entry, Arch))) -->
     [ 'Package ~w: no binary for architecture ~w'-[Entry, Arch] ].
@@ -1665,9 +1742,6 @@ prolog_message(backcomp(init_file_moved(FoundFile))) -->
 		 *          DEPRECATED		*
 		 *******************************/
 
-deprecated(Term) -->
-    prolog:deprecated(Term),
-    !.
 deprecated(set_prolog_stack(_Stack,limit)) -->
     [ 'set_prolog_stack/2: limit(Size) sets the combined limit.'-[], nl,
       'See https://www.swi-prolog.org/changes/stack-limit.html'
@@ -1692,6 +1766,69 @@ tripwire_context(_, Ctx) -->
     [ '~p'-[Ctx] ].
 
 
+		 /*******************************
+		 *     INTERNATIONALIZATION	*
+		 *******************************/
+
+:- create_prolog_flag(message_language, default, []).
+
+%!  message_lang(-Lang) is multi.
+%
+%   True when Lang is a language id  preferred for messages. Starts with
+%   the most specific language (e.g., `nl_BE`) and ends with `en`.
+
+message_lang(Lang) :-
+    current_message_lang(Lang0),
+    (   Lang0 == en
+    ->  Lang = en
+    ;   sub_atom(Lang0, 0, _, _, en_)
+    ->  longest_id(Lang0, Lang)
+    ;   (   longest_id(Lang0, Lang)
+        ;   Lang = en
+        )
+    ).
+
+longest_id(Lang, Id) :-
+    split_string(Lang, "_-", "", [H|Components]),
+    longest_prefix(Components, Taken),
+    atomic_list_concat([H|Taken], '_', Id).
+
+longest_prefix([H|T0], [H|T]) :-
+    longest_prefix(T0, T).
+longest_prefix(_, []).
+
+%!  current_message_lang(-Lang) is det.
+%
+%   Get the current language for messages.
+
+current_message_lang(Lang) :-
+    (   current_prolog_flag(message_language, Lang0),
+        Lang0 \== default
+    ->  Lang = Lang0
+    ;   os_user_lang(Lang0)
+    ->  clean_encoding(Lang0, Lang1),
+        set_prolog_flag(message_language, Lang1),
+        Lang = Lang1
+    ;   Lang = en
+    ).
+
+os_user_lang(Lang) :-
+    current_prolog_flag(windows, true),
+    win_get_user_preferred_ui_languages(name, [Lang|_]).
+os_user_lang(Lang) :-
+    catch(setlocale(messages, _, ''), _, fail),
+    setlocale(messages, Lang, Lang).
+os_user_lang(Lang) :-
+    getenv('LANG', Lang).
+
+
+clean_encoding(Lang0, Lang) :-
+    (   sub_atom(Lang0, A, _, _, '.')
+    ->  sub_atom(Lang0, 0, A, _, Lang)
+    ;   Lang = Lang0
+    ).
+
+
 		 /*******************************
 		 *        DEFAULT THEME		*
 		 *******************************/
@@ -1854,22 +1991,27 @@ msg_property(Kind, prefix(Prefix)) :-
 msg_property(_, prefix('~N')) :- !.
 msg_property(query, stream(user_output)) :- !.
 msg_property(_, stream(user_error)) :- !.
-msg_property(error,
+msg_property(error, tag('ERROR')).
+msg_property(warning, tag('Warning')).
+msg_property(Level,
              location_prefix(File:Line,
-                             '~NERROR: ~w:~d:'-[File,Line],
-                             '~NERROR:    ')) :- !.
-msg_property(warning,
-             location_prefix(File:Line,
-                             '~NWarning: ~w:~d:'-[File,Line],
-                             '~NWarning:    ')) :- !.
+                             Prefix1-[File,Line],
+                             PrefixC)) :-
+    include_msg_location(Level),
+    msg_property(Level, tag(Tag)),
+    atomics_to_string(['~N', Tag, ': ~w:~d:'], Prefix1),
+    atomics_to_string(['~N', Tag, ':    '  ], PrefixC).
 msg_property(error,   wait(0.1)) :- !.
 
+include_msg_location(warning).
+include_msg_location(error).
+
 msg_prefix(debug(_), Prefix) :-
     msg_context('~N% ', Prefix).
-msg_prefix(warning, Prefix) :-
-    msg_context('~NWarning: ', Prefix).
-msg_prefix(error, Prefix) :-
-    msg_context('~NERROR: ', Prefix).
+msg_prefix(Level, Prefix) :-
+    msg_property(Level, tag(Tag)),
+    atomics_to_string(['~N', Tag, ': '], Prefix0),
+    msg_context(Prefix0, Prefix).
 msg_prefix(informational, '~N% ').
 msg_prefix(information,   '~N% ').
 
