@@ -41,11 +41,13 @@
             print_prolog_backtrace/3,   % +Stream, +Stack, +Options
             backtrace/1                 % +MaxDepth
           ]).
-:- use_module(library(prolog_clause)).
-:- use_module(library(debug)).
-:- use_module(library(error)).
-:- use_module(library(lists)).
-:- use_module(library(option)).
+:- autoload(library(debug),[debug/3]).
+:- autoload(library(error),[must_be/2]).
+:- autoload(library(lists),[nth1/3,append/3]).
+:- autoload(library(option),[option/2,option/3,merge_options/3]).
+:- autoload(library(prolog_clause),
+	    [clause_name/2,predicate_name/2,clause_info/4]).
+
 
 :- dynamic stack_guard/1.
 :- multifile stack_guard/1.
@@ -101,10 +103,10 @@ decorate uncaught exceptions:
 %
 %     * frame(+Frame)
 %     Start at Frame instead of the current frame.
-%     * goal_depth(+Depth)
+%     * goal_term_depth(+Depth)
 %     If Depth > 0, include a shallow copy of the goal arguments
 %     into the stack.  Default is set by the Prolog flag
-%     `backtrace_goal_depth`, set to `2` initially, showing the
+%     `backtrace_goal_depth`, set to `3` initially, showing the
 %     goal and toplevel of any argument.
 %     * guard(+Guard)
 %     Do not show stack frames above Guard.  See stack_guard/1.
