@@ -14,7 +14,6 @@ from pathlib import PurePath, PurePosixPath, PureWindowsPath, Path
 import subprocess
 from contextlib import suppress
 import tempfile
-import traceback
 
 
 # From: https://eli.thegreenplace.net/2011/08/02/python-unit-testing-parametrized-test-cases/
@@ -846,150 +845,139 @@ class TestPrologMQI(ParametrizedTestCase):
 
     def test_server_options_and_shutdown(self):
         global secondsTimeoutForThreadExit
-        try:
-            with PrologMQI(
-                self.launchServer,
-                self.serverPort,
-                self.password,
-                self.useUnixDomainSocket,
-                prolog_path=self.prologPath,
-            ) as server:
-                with server.create_thread() as monitorThread:
-                    # Record the threads that are running, but give a pause so any threads created by the server on startup
-                    # can get closed down
-                    initialThreads = self.thread_list(monitorThread)
-                    socketPort = 4250
+        with PrologMQI(
+            self.launchServer,
+            self.serverPort,
+            self.password,
+            self.useUnixDomainSocket,
+            prolog_path=self.prologPath,
+        ) as server:
+            with server.create_thread() as monitorThread:
+                # Record the threads that are running, but give a pause so any threads created by the server on startup
+                # can get closed down
+                initialThreads = self.thread_list(monitorThread)
+                socketPort = 4250
 
-                    # password() should be used if supplied.
+                # password() should be used if supplied.
+                result = monitorThread.query(
+                    "mqi_start([port(Port), password(testpassword), server_thread(ServerThreadID)])"
+                )
+                serverThreadID = result[0]["ServerThreadID"]
+                port = result[0]["Port"]
+                with PrologMQI(
+                    launch_mqi=False,
+                    port=port,
+                    password="testpassword",
+                    prolog_path=self.prologPath,
+                ) as newServer:
+                    with newServer.create_thread() as prologThread:
+                        result = prologThread.query("true")
+                        self.assertEqual(result, True)
+                result = monitorThread.query(f"mqi_stop({serverThreadID})")
+                self.assertEqual(result, True)
+                afterShutdownThreads = self.thread_list(monitorThread)
+                self.wait_for_new_threads_exit(monitorThread, initialThreads, afterShutdownThreads, secondsTimeoutForThreadExit)
+
+                if os.name != "nt":
+                    # unixDomainSocket() should be used if supplied (non-windows).
+                    socketPath = mkdtemp()
+                    unixDomainSocket = PrologMQI.unix_domain_socket_file(socketPath)
                     result = monitorThread.query(
-                        "mqi_start([port(Port), password(testpassword), server_thread(ServerThreadID)])"
+                        f"mqi_start([unix_domain_socket('{unixDomainSocket}'), password(testpassword), server_thread(ServerThreadID)])"
                     )
                     serverThreadID = result[0]["ServerThreadID"]
-                    port = result[0]["Port"]
                     with PrologMQI(
                         launch_mqi=False,
-                        port=port,
+                        unix_domain_socket=unixDomainSocket,
                         password="testpassword",
                         prolog_path=self.prologPath,
                     ) as newServer:
                         with newServer.create_thread() as prologThread:
                             result = prologThread.query("true")
                             self.assertEqual(result, True)
-                    result = monitorThread.query(f"mqi_stop({serverThreadID})")
+                    result = monitorThread.query(
+                        f"mqi_stop({serverThreadID})"
+                    )
                     self.assertEqual(result, True)
                     afterShutdownThreads = self.thread_list(monitorThread)
                     self.wait_for_new_threads_exit(monitorThread, initialThreads, afterShutdownThreads, secondsTimeoutForThreadExit)
+                    assert not os.path.exists(unixDomainSocket)
 
-                    if os.name != "nt":
-                        # unixDomainSocket() should be used if supplied (non-windows).
-                        socketPath = mkdtemp()
-                        unixDomainSocket = PrologMQI.unix_domain_socket_file(socketPath)
-                        result = monitorThread.query(
-                            f"mqi_start([unix_domain_socket('{unixDomainSocket}'), password(testpassword), server_thread(ServerThreadID)])"
-                        )
-                        serverThreadID = result[0]["ServerThreadID"]
-                        with PrologMQI(
-                            launch_mqi=False,
-                            unix_domain_socket=unixDomainSocket,
-                            password="testpassword",
-                            prolog_path=self.prologPath,
-                        ) as newServer:
-                            with newServer.create_thread() as prologThread:
-                                result = prologThread.query("true")
-                                self.assertEqual(result, True)
-                        result = monitorThread.query(
-                            f"mqi_stop({serverThreadID})"
-                        )
-                        self.assertEqual(result, True)
-                        afterShutdownThreads = self.thread_list(monitorThread)
-                        self.wait_for_new_threads_exit(monitorThread, initialThreads, afterShutdownThreads, secondsTimeoutForThreadExit)
-                        assert not os.path.exists(unixDomainSocket)
-
-                        # unixDomainSocket() should be generated if asked for (non-windows).
-                        result = monitorThread.query(
-                            "mqi_start([unix_domain_socket(Socket), password(testpassword), server_thread(ServerThreadID)])"
-                        )
-                        serverThreadID = result[0]["ServerThreadID"]
-                        unixDomainSocket = result[0]["Socket"]
-                        with PrologMQI(
-                            launch_mqi=False,
-                            unix_domain_socket=unixDomainSocket,
-                            password="testpassword",
-                            prolog_path=self.prologPath,
-                        ) as newServer:
-                            with newServer.create_thread() as prologThread:
-                                result = prologThread.query("true")
-                                self.assertEqual(result, True)
-                        result = monitorThread.query(
-                            f"mqi_stop({serverThreadID})"
-                        )
-                        self.assertEqual(result, True)
-                        afterShutdownThreads = self.thread_list(monitorThread)
-                        self.wait_for_new_threads_exit(monitorThread, initialThreads, afterShutdownThreads, secondsTimeoutForThreadExit)
-                        # Temp Socket should not exist
-                        assert not os.path.exists(unixDomainSocket)
-                        # Neither should Temp directory
-                        assert not os.path.exists(Path(unixDomainSocket).parent)
-
-                    # runServerOnThread(false) should block until the server is shutdown.
-                    # Create a new connection that we block starting a new server
-                    with server.create_thread() as blockedThread:
-                        blockedThread.query_async(
-                            "mqi_start([port({}), password(testpassword), run_server_on_thread(false), server_thread(testServerThread)])".format(
-                                socketPort
-                            )
-                        )
-                        # Wait for the server to start
-                        sleep(1)
-
-                        # Make sure we are still blocked
-                        exceptionCaught = False
-                        try:
-                            blockedThread.query_async_result(wait_timeout_seconds=0)
-                        except PrologResultNotAvailableError:
-                            exceptionCaught = True
-                        assert exceptionCaught
-
-                        # Ensure the server started by sending it a query
-                        with PrologMQI(
-                            launch_mqi=False,
-                            port=socketPort,
-                            password="testpassword",
-                            prolog_path=self.prologPath,
-                        ) as newServer:
-                            with newServer.create_thread() as prologThread:
-                                result = prologThread.query("true")
-                                self.assertEqual(result, True)
-                        # Make sure we are still blocked
-                        exceptionCaught = False
-                        try:
-                            blockedThread.query_async_result(wait_timeout_seconds=0)
-                        except PrologResultNotAvailableError:
-                            exceptionCaught = True
-                        assert exceptionCaught
-
-                        # Now shut it down by cancelling the query and running stop
-                        blockedThread.cancel_query_async()
-                        result = monitorThread.query(
-                            f"mqi_stop({blockedThread.communication_thread_id})"
-                        )
-                        self.assertEqual(result, True)
-
-                    # And make sure all the threads went away
+                    # unixDomainSocket() should be generated if asked for (non-windows).
+                    result = monitorThread.query(
+                        "mqi_start([unix_domain_socket(Socket), password(testpassword), server_thread(ServerThreadID)])"
+                    )
+                    serverThreadID = result[0]["ServerThreadID"]
+                    unixDomainSocket = result[0]["Socket"]
+                    with PrologMQI(
+                        launch_mqi=False,
+                        unix_domain_socket=unixDomainSocket,
+                        password="testpassword",
+                        prolog_path=self.prologPath,
+                    ) as newServer:
+                        with newServer.create_thread() as prologThread:
+                            result = prologThread.query("true")
+                            self.assertEqual(result, True)
+                    result = monitorThread.query(
+                        f"mqi_stop({serverThreadID})"
+                    )
+                    self.assertEqual(result, True)
                     afterShutdownThreads = self.thread_list(monitorThread)
                     self.wait_for_new_threads_exit(monitorThread, initialThreads, afterShutdownThreads, secondsTimeoutForThreadExit)
+                    # Temp Socket should not exist
+                    assert not os.path.exists(unixDomainSocket)
+                    # Neither should Temp directory
+                    assert not os.path.exists(Path(unixDomainSocket).parent)
 
-                    # Launching this library itself and stopping in the debugger tests writeConnectionValues() and ignoreSigint and haltOnConnectionFailure internal features automatically
+                # runServerOnThread(false) should block until the server is shutdown.
+                # Create a new connection that we block starting a new server
+                with server.create_thread() as blockedThread:
+                    blockedThread.query_async(
+                        "mqi_start([port({}), password(testpassword), run_server_on_thread(false), server_thread(testServerThread)])".format(
+                            socketPort
+                        )
+                    )
+                    # Wait for the server to start
+                    sleep(1)
 
-        except Exception as e:
-            if self.failOnUnlikely:
-                # Rethrow the exception if we are configured to fail on unlikely tests failures
-                raise
-            else:
-                stackTrace = ''.join(traceback.format_exception(etype=type(e), value=e, tb=e.__traceback__))
-                print(
-                    f"WARNING: {e} at {stackTrace}.\n This can happen if the system is heavily loaded and is thus a warning by default. To turn this into a failure set the environment variable 'SWIPL_TEST_FAIL_ON_UNLIKELY=y'."
-                )
+                    # Make sure we are still blocked
+                    exceptionCaught = False
+                    try:
+                        blockedThread.query_async_result(wait_timeout_seconds=0)
+                    except PrologResultNotAvailableError:
+                        exceptionCaught = True
+                    assert exceptionCaught
+
+                    # Ensure the server started by sending it a query
+                    with PrologMQI(
+                        launch_mqi=False,
+                        port=socketPort,
+                        password="testpassword",
+                        prolog_path=self.prologPath,
+                    ) as newServer:
+                        with newServer.create_thread() as prologThread:
+                            result = prologThread.query("true")
+                            self.assertEqual(result, True)
+                    # Make sure we are still blocked
+                    exceptionCaught = False
+                    try:
+                        blockedThread.query_async_result(wait_timeout_seconds=0)
+                    except PrologResultNotAvailableError:
+                        exceptionCaught = True
+                    assert exceptionCaught
+
+                    # Now shut it down by cancelling the query and running stop
+                    blockedThread.cancel_query_async()
+                    result = monitorThread.query(
+                        f"mqi_stop({blockedThread.communication_thread_id})"
+                    )
+                    self.assertEqual(result, True)
+
+                # And make sure all the threads went away
+                afterShutdownThreads = self.thread_list(monitorThread)
+                self.wait_for_new_threads_exit(monitorThread, initialThreads, afterShutdownThreads, secondsTimeoutForThreadExit)
+
+                # Launching this library itself and stopping in the debugger tests writeConnectionValues() and ignoreSigint and haltOnConnectionFailure internal features automatically
 
     def test_server_options_and_shutdown_slow(self):
         global secondsTimeoutForThreadExit
@@ -1272,20 +1260,6 @@ class TestPrologMQI(ParametrizedTestCase):
                     f"Best Time to run {iterations} iterations of the Prolog query `true`: {bestResult}"
                 )
 
-    # Run a simple query 1000 times to test for leaks
-    def skip_test_launch_stress(self):
-        for index in range(0, 10000):
-            print(index)
-            with PrologMQI(
-                self.launchServer,
-                self.serverPort,
-                self.password,
-                self.useUnixDomainSocket,
-                prolog_path=self.prologPath,
-            ) as server:
-                with PrologThread(server) as prolog_thread:
-                    prolog_thread.query("true")
-
 
 def run_tcpip_performance_tests(suite):
     suite.addTest(TestPrologMQI("skip_test_protocol_overhead"))
@@ -1316,10 +1290,8 @@ def load_tests(loader, standard_tests, pattern):
     # run_unix_domain_sockets_performance_tests(suite)
 
     # Tests a specific test
-    # suite.addTest(TestPrologMQI('skip_test_launch_stress'))
-    # return suite
-
-    # Tests a specific test with parameters set
+    # suite.addTest(TestPrologMQI('test_server_options_and_shutdown'))
+    # socketPath = os.path.dirname(os.path.realpath(__file__))
     # suite.addTest(ParametrizedTestCase.parametrize(TestPrologMQI, test_item_name="test_debugging_options", launchServer=False,
     #                                                serverPort=4242, password="test"))
 
