@@ -34,8 +34,9 @@
 */
 
 #define _GNU_SOURCE
-#include "pl-incl.h"
-#include "os/pl-cstack.h"
+#include "../pl-incl.h"
+#include "pl-cstack.h"
+#include "../pl-setup.h"
 #include <time.h>
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -370,7 +371,7 @@ addr2line(const char *fname, uintptr_t offset, char *buf, size_t size)
 
       *o = '\0';
 
-      fclose(fd);
+      pclose(fd);
       return o > buf;
     }
   }
@@ -432,7 +433,7 @@ print_backtrace(int last)		/* 1..SAVE_TRACES */
 }
 
 
-void
+static void
 bstore_print_backtrace_named(btrace *bt, const char *why)
 { if ( bt )
   { int me = bt->current-1;
