@@ -215,7 +215,7 @@ static int	update_column(int, Char);
 static bool	do_format(IOSTREAM *fd, PL_chars_t *fmt,
 			  int ac, term_t av, Module m);
 static void	distribute_rubber(struct rubber *, int, int);
-static int	emit_rubber(format_state *state);
+static WUNUSED int emit_rubber(format_state *state);
 
 
 		/********************************
@@ -378,6 +378,66 @@ get_chr_from_text(const PL_chars_t *t, int index)
 }
 
 
+typedef struct sub_state
+{ char          buf[BUFSIZE];
+  char         *str;
+  size_t        bufsize;
+  format_state *fstate;
+  IOSTREAM     *old_stream;
+} sub_state;
+
+static int
+prepare_sub_format(sub_state *state, format_state *fstate, IOSTREAM *fd)
+{ state->fstate = fstate;
+
+  if ( !fstate->pending_rubber &&
+       fd->position &&
+       fd->position->linepos == fstate->column )
+  { state->old_stream = Scurout;
+
+    Scurout = fd;
+  } else
+  { state->old_stream = NULL;
+    state->str        = state->buf;
+    state->bufsize    = sizeof(state->buf);
+
+    tellString(&state->str, &state->bufsize, ENC_UTF8);
+    if ( true(fd, SIO_ISATTY) )
+      set(Scurout, SIO_ISATTY);
+  }
+
+  return TRUE;
+}
+
+static int
+end_sub_format(sub_state *state, int rc)
+{ int lp = Scurout->position->linepos;
+
+  if ( state->old_stream )
+  { Scurout = state->old_stream;
+
+    if ( rc )
+      state->fstate->column = lp;
+  } else
+  { int lp = Scurout->position->linepos;
+
+    toldString();
+    if ( rc )
+    { int c0 = state->fstate->column;
+      rc = oututf8(state->fstate, state->str, state->bufsize);
+      state->fstate->column = c0 + lp;
+    }
+    if ( state->str != state->buf )
+      free(state->str);
+  }
+
+  return rc;
+}
+
+
+
+
+
 		/********************************
 		*       ACTUAL FORMATTING	*
 		********************************/
@@ -452,9 +512,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 	       (proc = lookupHTable(format_predicates, (void*)((intptr_t)c))) )
 	  { size_t arity;
 	    term_t av;
-	    char buf[BUFSIZE];
-	    char *str = buf;
-	    size_t bufsize = BUFSIZE;
+	    sub_state sstate;
 	    int i;
 
 	    PL_predicate_info(proc, NULL, &arity, NULL);
@@ -471,13 +529,11 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 	      SHIFT;
 	    }
 
-	    tellString(&str, &bufsize, ENC_UTF8);
+	    if ( !(rc=prepare_sub_format(&sstate, &state, fd)) )
+	      goto out;
 	    rc = PL_call_predicate(NULL, PL_Q_PASS_EXCEPTION, proc, av);
-	    toldString();
-	    if ( rc )
-	      rc = oututf8(&state, str, bufsize);
-	    if ( str != buf )
-	      free(str);
+	    rc = end_sub_format(&sstate, rc);
+
 	    if ( !rc )
 	      goto out;
 
@@ -645,8 +701,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  break;
 		}
 		{ Func f;
-		  char buf[BUFSIZE];
-		  char *str;
+		  sub_state sstate;
 
 	      case 'k':			/* write_canonical */
 		  f = pl_write_canonical;
@@ -662,121 +717,51 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  pl_common:
 
 		  NEED_ARG;
-		  if ( state.pending_rubber )
-		  { size_t bufsize = BUFSIZE;
 
-		    str = buf;
-		    tellString(&str, &bufsize, ENC_UTF8);
-		    rc = (*f)(argv);
-		    toldString();
-		    if ( rc )
-		      rc = oututf8(&state, str, bufsize);
-		    if ( str != buf )
-		      free(str);
-		    if ( !rc )
-		      goto out;
-		  } else
-		  { if ( fd->position &&
-			 fd->position->linepos == state.column )
-		    { IOSTREAM *old = Scurout;
+		  if ( !(rc=prepare_sub_format(&sstate, &state, fd)) )
+		    goto out;
+		  rc = (int)(*f)(argv);
+		  rc = end_sub_format(&sstate, rc);
 
-		      Scurout = fd;
-		      rc = (int)(*f)(argv);
-		      Scurout = old;
-		      if ( !rc )
-			goto out;
-
-		      state.column = fd->position->linepos;
-		    } else
-		    { size_t bufsize = BUFSIZE;
-
-		      str = buf;
-		      tellString(&str, &bufsize, ENC_UTF8);
-		      rc = (*f)(argv);
-		      toldString();
-		      if ( rc )
-			rc = oututf8(&state, str, bufsize);
-		      if ( str != buf )
-			free(str);
-		      if ( !rc )
-			goto out;
-		    }
-		  }
 		  SHIFT;
 		  here++;
 		  break;
 		}
 	      case 'W':			/* write_term(Value, Options) */
-	       { char buf[BUFSIZE];
-		 char *str;
+	       { sub_state sstate;
 
 		 if ( argc < 2 )
 		 { FMT_ERROR("not enough arguments");
 		 }
-		 if ( state.pending_rubber )
-		  { size_t bufsize = BUFSIZE;
 
-		    str = buf;
-		    tellString(&str, &bufsize, ENC_UTF8);
-		    rc = (int)pl_write_term(argv, argv+1);
-		    toldString();
-		    if ( rc )
-		      rc = oututf8(&state, str, bufsize);
-		    if ( str != buf )
-		      free(str);
-		    if ( !rc )
-		      goto out;
-		  } else
-		  { if ( fd->position &&
-			 fd->position->linepos == state.column )
-		    { IOSTREAM *old = Scurout;
+		 if ( !(rc=prepare_sub_format(&sstate, &state, fd)) )
+		   goto out;
+		 rc = (int)pl_write_term(argv, argv+1);
+		 rc = end_sub_format(&sstate, rc);
 
-		      Scurout = fd;
-		      rc = (int)pl_write_term(argv, argv+1);
-		      Scurout = old;
-		      if ( !rc )
-			goto out;
+		 if ( !rc )
+		   goto out;
 
-		      state.column = fd->position->linepos;
-		    } else
-		    { size_t bufsize = BUFSIZE;
-
-		      str = buf;
-		      tellString(&str, &bufsize, ENC_UTF8);
-		      rc = (int)pl_write_term(argv, argv+1);
-		      toldString();
-		      if ( rc )
-			rc = oututf8(&state, str, bufsize);
-		      if ( str != buf )
-			free(str);
-		      if ( !rc )
-			goto out;
-		    }
-		  }
-		  SHIFT;
-		  SHIFT;
-		  here++;
-		  break;
+		 SHIFT;
+		 SHIFT;
+		 here++;
+		 break;
 	       }
 	      case '@':
-	        { char buf[BUFSIZE];
-		  char *str = buf;
-		  size_t bufsize = BUFSIZE;
-		  term_t ex = 0;
+	        { term_t ex = 0;
+		  sub_state sstate;
 
 		  if ( argc < 1 )
 		  { FMT_ERROR("not enough arguments");
 		  }
-		  tellString(&str, &bufsize, ENC_UTF8);
+
+		  if ( !(rc=prepare_sub_format(&sstate, &state, fd)) )
+		    goto out;
 		  rc = callProlog(m, argv, PL_Q_CATCH_EXCEPTION, &ex);
-		  toldString();
-		  if ( rc )
-		    rc = oututf8(&state, str, bufsize);
-		  if ( str != buf )
-		    free(str);
+		  rc = end_sub_format(&sstate, rc);
 
 		  if ( !rc )
-		  { if ( ex )
+		  { if ( ex && !PL_exception(0) )
 		      rc = PL_raise_exception(ex);
 		    goto out;
 		  }
@@ -821,14 +806,22 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		}
 	      case '|':			/* set tab */
 		{ int stop;
+		  int nl_and_reindent;
 
 		  if ( arg == DEFAULT )
 		    arg = state.column;
+		  /*FALLTHROUGH*/
 	      case '+':			/* tab relative */
 		  if ( arg == DEFAULT )
 		    arg = 8;
 		  stop = (c == '+' ? tab_stop + arg : arg);
 
+		  if ( stop < state.column && mod_colon )
+		    nl_and_reindent = state.pending_rubber ?
+						state.rub[state.pending_rubber-1].pad : ' ';
+		  else
+		    nl_and_reindent = 0;
+
 		  if ( state.pending_rubber == 0 ) /* nothing to distribute */
 		  { state.rub[0].where = state.buffered;
 		    state.rub[0].pad = ' ';
@@ -837,7 +830,26 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
 		  distribute_rubber(state.rub,
 				    state.pending_rubber,
 				    stop - state.column);
-		  emit_rubber(&state);
+		  if ( !(rc=emit_rubber(&state)) )
+		    goto out;
+
+		  if ( nl_and_reindent )
+		  { if ( Sputcode('\n', state.out) < 0 )
+		    { rc = FALSE;
+		      goto out;
+		    }
+		    state.column = update_column(state.column, '\n');
+
+		    state.rub[0].where = state.buffered;
+		    state.rub[0].pad = nl_and_reindent;
+		    state.pending_rubber++;
+
+		    distribute_rubber(state.rub,
+				      state.pending_rubber,
+				      stop - state.column);
+		    if ( !(rc=emit_rubber(&state)) )
+		      goto out;
+		  }
 
 		  state.column = tab_stop = stop;
 		  here++;
@@ -866,7 +878,9 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv, Module m)
   }
 
   if ( state.pending_rubber )		/* not closed ~t: flush out */
-    emit_rubber(&state);
+  { if ( !(rc=emit_rubber(&state)) )
+      goto out;
+  }
   if ( argc != 0 )
     FMT_ERROR("too many arguments");
 
@@ -897,7 +911,7 @@ distribute_rubber(struct rubber *r, int rn, int space)
 }
 
 
-static int
+static WUNUSED int
 emit_rubber(format_state *state)
 { const char *s = baseBuffer(&state->buffer, char);
   const char *e = &s[entriesBuffer(&state->buffer, char)];
