@@ -3,10 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2021, University of Amsterdam,
+    Copyright (c)  1985-2020, University of Amsterdam,
                               VU University Amsterdam
 			      CWI, Amsterdam
-			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -131,15 +130,8 @@
  */
 #define COMMON(type) type
 
-#include "pl-macros.h"
-
-/* C11 gives us the _Static_assert operator, let's make it a little nicer.
- * Accept either static_assert(cond, "message") or static_assertion(cond). */
-#define static_assert(condition, message) _Static_assert(condition, message)
-#define static_assertion(condition) _Static_assert(condition, "Assertion failed: ("#condition") [expansion: " A_STRINGIFY(condition) "]")
-
 #include "pl-builtin.h"
-
+#include "pl-macros.h"
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 		      PROLOG SYSTEM OPTIONS
@@ -205,13 +197,6 @@ handy for it someone wants to add a data type to the system.
       Default for the `prefer_rationals` flag.
   O_RATIONAL_SYNTAX
       Default support for rational syntax (RAT_NATURAL or RAT_COMPAT)
-  O_SIGNALS
-      Include OS-level signal-handling code. This does not affect any
-      virtual Prolog "signals" defined above SIG_PROLOG_OFFSET, but
-      SIG_ALERT loses its "interrupt a blocking system call" semantics
-      when this is disabled. Presently this also disables support for
-      Prolog-facing signal handling (on_signal/3, etc) but that may
-      change in the future.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #define O_COMPILE_OR		1
@@ -364,20 +349,9 @@ typedef _sigset_t sigset_t;
 #include <stdarg.h>
 #include <limits.h>
 
-#ifdef HAVE_SIGNAL_H 
+#ifdef HAVE_SIGNAL
 #include <signal.h>
 #endif
-#if !O_SIGNALS
-/* Pretend like we don't have access to any of these system calls */
-# undef HAVE_SIGNAL
-# undef HAVE_SIGPROCMASK
-# undef HAVE_SIGSETMASK
-# undef HAVE_SIGGETMASK
-# undef HAVE_SIGACTION
-# undef HAVE_SIGSET
-# undef HAVE_SIGBLOCK
-# undef HAVE_SIGALTSTACK
-#endif /* O_SIGNALS */
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #else
@@ -589,11 +563,11 @@ sizes  of  the  hash  tables are defined.  Note that these should all be
 2^N.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define ATOMHASHSIZE		4096	/* global atom table */
+#define ATOMHASHSIZE		1024	/* global atom table */
 #define FUNCTORHASHSIZE		512	/* global functor table */
-#define PROCEDUREHASHSIZE	1024	/* predicates in module user */
+#define PROCEDUREHASHSIZE	256	/* predicates in module user */
 #define MODULEPROCEDUREHASHSIZE 16	/* predicates in other modules */
-#define MODULEHASHSIZE		128	/* global module table */
+#define MODULEHASHSIZE		16	/* global module table */
 #define PUBLICHASHSIZE		8	/* Module export table */
 #define FLAGHASHSIZE		16	/* global flag/3 table */
 
@@ -641,7 +615,6 @@ Magic for assertions.
 
 #define StackMagic(n)	((n) | 0x98765000)
 #define QID_MAGIC	StackMagic(1)	/* Query frame */
-#define QID_CMAGIC	StackMagic(2)	/* Closed Query frame */
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 			  PROLOG DATA REPRESENTATION
@@ -1747,11 +1720,6 @@ typedef struct exception_frame		/* PL_throw exception environments */
 } exception_frame;
 
 
-typedef struct queryRef
-{ PL_engine_t	engine;			/* Engine for the query */
-  uintptr_t	offset;			/* queryFrane offset in local stack */
-} *QueryRef;
-
 #define QF_NODEBUG		0x0001	/* debug-able query */
 #define QF_DETERMINISTIC	0x0002	/* deterministic success */
 #define	QF_INTERACTIVE		0x0004	/* interactive goal (prolog()) */
@@ -1783,7 +1751,6 @@ struct queryFrame
   Choice	saved_bfr;		/* Saved choice-point */
   LocalFrame	saved_ltop;		/* Saved lTop */
   QueryFrame	parent;			/* Parent queryFrame */
-  QueryRef	qid;			/* Opaque query id */
   struct choice	choice;			/* First (dummy) choice-point */
   LocalFrame	saved_environment;	/* Parent local-frame */
 					/* Do not put anything between */
@@ -2086,7 +2053,8 @@ Temporary store/restore pointers to make them safe over GC/shift
 			   } while(0)
 
 
-#define QidFromQuery(f)		((f)->qid)
+#define QueryFromQid(qid)	((QueryFrame) valTermRef(qid))
+#define QidFromQuery(f)		(consTermRef(f))
 #define QID_EXPORT_WAM_TABLE	(qid_t)(-1)
 
 
@@ -2100,22 +2068,17 @@ still have signals that trigger Prolog   housekeeping  events. These are
 not bound to operating system signal handling though.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define SIG_PROLOG_OFFSET	32	/* Start of Prolog signals - specified in docs */
-
 #if HAVE_SIGNAL
-
-#define MINSIGNAL		1	/* number of first signal */
-#define MAXSIGNAL		64	/* highest supported signal number */
+#define MAXSIGNAL		64	/* highest system signal number */
+#define SIG_PROLOG_OFFSET	32	/* Start of Prolog signals */
 
 #else /* HAVE_SIGNAL */
 
-#define MINSIGNAL		SIG_PROLOG_OFFSET /* No system signals, only Prolog sigs */
-#define MAXSIGNAL		(SIG_PROLOG_OFFSET+31)	/* we'd like this to fit in 32 bits */
+#define MAXSIGNAL		32	/* highest system signal number */
+#define SIG_PROLOG_OFFSET	1	/* Start of Prolog signals */
 
 #endif /* HAVE_SIGNAL */
 
-#define NUM_SIGNALS (MAXSIGNAL - MINSIGNAL + 1)
-
 #ifndef RETSIGTYPE
 #define RETSIGTYPE void
 #endif
@@ -2128,105 +2091,17 @@ typedef struct
   int	      flags;			/* PLSIG_*, defined in pl-setup.c */
 } sig_handler, *SigHandler;
 
-/* Declare numbers for the virtual signals, in their own domain. For now, these
- * just map to the numbers starting at SIG_PROLOG_OFFSET. Note, we could omit
- * some of these definitions based on preprocessor statements, but there are
- * few enough of them that it doesn't seem necessary.
- */
-typedef enum virtual_signum
-{ VSIG_ATOM_GC,
-  VSIG_GC,
-  VSIG_THREAD_SIGNAL,
-  VSIG_CLAUSE_GC,
-  VSIG_PLABORT,
-  VSIG_TUNE_GC,
-  VSIG_MAX
-} virtual_signum;
 
-#define NUM_VSIGS 6 /* Preprocessor can see this constant */
-static_assertion(NUM_VSIGS == VSIG_MAX); /* Make sure it matches the enum */
-static_assertion(NUM_SIGNALS >= VSIG_MAX && NUM_SIGNALS < 128); /* Sanity check, 128 is arbitrary */
-static_assertion(SIG_PROLOG_OFFSET >= MINSIGNAL && SIG_PROLOG_OFFSET + NUM_VSIGS <= MAXSIGNAL);
-
-/* Define the macros that map vsignals into the signal domain. */
 #ifdef O_ATOMGC
-#define SIG_ATOM_GC	  (SIG_PROLOG_OFFSET+VSIG_ATOM_GC)
+#define SIG_ATOM_GC	  (SIG_PROLOG_OFFSET+0)
 #endif
-#define SIG_GC		  (SIG_PROLOG_OFFSET+VSIG_GC)
+#define SIG_GC		  (SIG_PROLOG_OFFSET+1)
 #ifdef O_PLMT
-#define SIG_THREAD_SIGNAL (SIG_PROLOG_OFFSET+VSIG_THREAD_SIGNAL)
-#endif
-#define SIG_CLAUSE_GC	  (SIG_PROLOG_OFFSET+VSIG_CLAUSE_GC)
-#define SIG_PLABORT	  (SIG_PROLOG_OFFSET+VSIG_PLABORT)
-#define SIG_TUNE_GC	  (SIG_PROLOG_OFFSET+VSIG_TUNE_GC)
-
-/* The "search for a free signal" functionality of PL_sigaction starts after
- * the predefined VSIG numbers */
-#define SIG_USER_OFFSET	  (SIG_PROLOG_OFFSET+VSIG_MAX)
-
-/* Get a zero-based array index for this signal */
-#define SIGNAL_INDEX(sig)	((sig) - MINSIGNAL)
-/* Return the signal given an array index */
-#define SIGNAL_FROM_INDEX(idx)	((idx) + MINSIGNAL)
-/* Is this a valid signal number? */
-#define IS_VALID_SIGNAL(sig)	((sig) >= MINSIGNAL && (sig) <= MAXSIGNAL)
-/* Is this a virtual signal? */
-#define IS_VSIG(sig)		((sig) >= SIG_PROLOG_OFFSET)
-
-/* We want fast types for signal bitmasks; on a 64-bit arch this is probably the same as uint64_t */
-typedef uint_fast32_t		sigmask_t;
-
-/* How many bits can fit in a single sigmask_t? */
-#define SIGMASK_WIDTH		(sizeof(sigmask_t) * 8)
-/* How many sigmask_t's does it take to store all supported signals? */
-#define SIGMASK_WORDS		((NUM_SIGNALS + SIGMASK_WIDTH - 1) / SIGMASK_WIDTH)
-/* Which sigmask word is this signal in? */
-#define SIGMASK_WORD(sig)	(SIGNAL_INDEX(sig) / SIGMASK_WIDTH)
-/* Which bit of the given word? */
-#define SIGMASK_BIT(sig)	(SIGNAL_INDEX(sig) % SIGMASK_WIDTH)
-/* Single-bit mask corresponding to SIGMASK_BIT */
-#define SIGMASK_MASK(sig)	(((sigmask_t)1) << SIGMASK_BIT(sig))
-
-/* Obtain signal given sigmask word and bit */
-#define SIGNAL_FROM_SIGMASK(word, bit) SIGNAL_FROM_INDEX((word) * SIGMASK_WIDTH + (bit))
-
-typedef sigmask_t		wsigmask_t[SIGMASK_WORDS];
-
-/* Helper macros for interacting with sigmask_t arrays */
-#define WSIGMASK_ISSET(wm, sig)	(((wm)[SIGMASK_WORD(sig)] & SIGMASK_MASK(sig)) != 0)
-#define WSIGMASK_SET(wm, sig)	ATOMIC_OR(&(wm)[SIGMASK_WORD(sig)], SIGMASK_MASK(sig))
-#define WSIGMASK_CLEAR(wm, sig)	ATOMIC_AND(&(wm)[SIGMASK_WORD(sig)], ~SIGMASK_MASK(sig))
-
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Guard against C-stack overflows. This is   done  for POSIX systems using
-the  alternative  signal  stack.  The  signal    handling   is  done  by
-alt_segv_handler().
-
-Note that we  use  setjmp()  rather   than  sigsetjmp().  The  latter is
-simpler, but a lot slower as it  implies   a  system  call. We assume no
-other signals are involved and unblock SIGSEGV by hand.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-#if defined(HAVE_SIGALTSTACK) && !defined(__SANITIZE_ADDRESS__)
-#define O_C_STACK_GUARDED 1
-#define C_STACK_OVERFLOW_GUARDED(rc, code, cleanup) \
-	do						\
-	{ LD->signal.sig_critical = TRUE;		\
-	  if ( setjmp(LD->signal.context) )		\
-	  { cleanup;					\
-	    unblockSignal(SIGSEGV);			\
-	    rc = PL_resource_error("c_stack");		\
-	  } else					\
-	  { rc = code;					\
-	  }						\
-	  LD->signal.sig_critical = FALSE;		\
-	} while(0)
-#else
-#define C_STACK_OVERFLOW_GUARDED(rc, code, cleanup) \
-	do						\
-	{ rc = code;						\
-	} while(0)
+#define SIG_THREAD_SIGNAL (SIG_PROLOG_OFFSET+2)
 #endif
+#define SIG_CLAUSE_GC	  (SIG_PROLOG_OFFSET+3)
+#define SIG_PLABORT	  (SIG_PROLOG_OFFSET+4)
+#define SIG_TUNE_GC	  (SIG_PROLOG_OFFSET+5)
 
 
 		 /*******************************
@@ -2401,8 +2276,6 @@ typedef struct
 	  if ( save ) \
 	  { _savedf = LD->var_names.numbervars_frame; \
 	    LD->var_names.numbervars_frame = PL_open_foreign_frame(); \
-	  } else \
-	  { _savedf = 0; /* avoid (false) uninitialized */  \
 	  }
 #define END_NUMBERVARS(save) \
           if ( save ) \
@@ -2757,37 +2630,6 @@ with any luck LTO could address that.
 # define ALLOC_INLINE
 #endif
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Helpers for weak function references, where we want our behavior to change
-based on if we've been linked against a specific library or not. To use a
-weak function, you should (e.g. for weak-importing malloc):
-
-1. WEAK_DECLARE(void *, malloc, (size_t size)); at .c file scope
-2. WEAK_IMPORT(malloc) in startup code (returns success)
-3. if (WEAK_FUNC(malloc)) to check if the import succeeded
-4a. WEAK_FUNC(malloc)(sizeof(x)) to call the function, if import worked, or
-4b. WEAK_TRY_CALL(malloc, sizeof(x)) to combine check and call.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-static inline int __ptr_to_bool(const void *ptr) { return ptr != NULL; }
-#if HAVE_WEAK_ATTRIBUTE
-# define WEAK_DECLARE(RType, Name, Params) \
-	extern RType __attribute__((weak)) Name Params
-# define WEAK_IMPORT(Name) __ptr_to_bool(&Name)
-# define WEAK_FUNC(Name) (&Name)
-#elif defined(HAVE_DLOPEN) || defined(HAVE_SHL_LOAD) || defined(EMULATE_DLOPEN)
-# define WEAK_DECLARE(RType, Name, Params) \
-	static RType (*wf##Name) Params = NULL
-# define WEAK_IMPORT(Name) __ptr_to_bool((wf##Name = PL_dlsym(NULL, #Name)))
-# define WEAK_FUNC(Name) (wf##Name)
-#else
-# define WEAK_DECLARE(RType, Name, Params) \
-	extern RType Name Params
-# define WEAK_IMPORT(Name) 0
-# define WEAK_FUNC(Name) 0
-#endif
-#define WEAK_TRY_CALL(Name, ...) (WEAK_FUNC(Name) != NULL ? WEAK_FUNC(Name)(__VA_ARGS__) : 0)
-
 #include "pl-util.h"			/* (Debug) utilities */
 #include "pl-alloc.h"			/* Allocation primitives */
 #include "pl-init.h"			/* Declarations needed by pl-init.c */
@@ -2799,10 +2641,10 @@ static inline int __ptr_to_bool(const void *ptr) { return ptr != NULL; }
 #include "os/pl-locale.h"		/* Locale objects */
 #include "os/pl-file.h"			/* Stream management */
 #include "pl-global.h"			/* global data */
-#include "pl-hash.h"			/* Murmurhash function */
 #include "pl-inline.h"			/* Inline facilities */
 #include "pl-privitf.h"			/* private foreign interface */
 #include "os/pl-text.h"			/* text manipulation */
+#include "pl-hash.h"			/* Murmurhash function */
 #include "os/pl-option.h"		/* Option processing */
 #include "os/pl-files.h"		/* File management */
 #include "os/pl-string.h"		/* Basic string functions */
